# -*- coding: utf-8 -*-
import json
import os
import collections 

class StudyData():
    _instance=None
    """Class to write and save informations about process in the json"""
    def __init__(self):
        self.name_study = "Study"
        self.study_directory = ""
        self.compteur_run_process={}
        self.runs=collections.OrderedDict()

    @staticmethod
    def get_instance():
        if StudyData._instance is None:
            StudyData._instance=StudyData()
            return StudyData._instance
        else:
            return StudyData._instance

    """Save on json with OrderedDict"""
    def save(self):
        self.dico=collections.OrderedDict([('name_study',self.name_study),('study_directory',self.study_directory),('compteur_run_process',self.compteur_run_process),('runs',self.runs)])
        json_string = json.dumps(self.dico, indent=4, separators=(',', ': '))
        with open(os.path.join(self.study_directory,self.name_study+'.json'), 'w') as f:
            f.write(unicode(json_string))

    """Load and put on self.__dict__ OrderedDict"""
    def load(self):
        try:
            with open(os.path.join(self.study_directory,self.name_study+'.json'), 'r') as json_data:
                self.__dict__ = json.load(json_data,object_pairs_hook=collections.OrderedDict)        
        #No file to load
        except IOError:
            pass

    """Get number of run process and iterate"""
    def inc_nb_run_process(self,process_name):
        if self.compteur_run_process.has_key(process_name):
            valeur=self.compteur_run_process[process_name]
            self.compteur_run_process[process_name]=valeur+1
        else:
            self.compteur_run_process[process_name]=1




'''
Functions to manage private/public keys encryption.
This module needs Crypto module.
'''

from Crypto.PublicKey import RSA
from base64 import b64decode

def generate_RSA(bits=2048):
  '''
  Generate an RSA keypair with an exponent of 65537 in PEM format
  param: bits The key length in bits
  Return private key and public key
  '''
  new_key = RSA.generate(bits)
  public_key = new_key.publickey().exportKey("PEM")
  private_key = new_key.exportKey("PEM")
  return private_key, public_key

def encrypt_RSA(public_key_loc, message):
  '''
  param: public_key_loc Path to public key
  param: message String to be encrypted
  return base64 encoded encrypted string
  '''
  key = open(public_key_loc, "r").read()
  rsakey = RSA.importKey(key)
  encrypted = rsakey.encrypt(message,None)[0]
  return encrypted.encode('base64')


def decrypt_RSA(private_key_loc, package):
  '''
  param: public_key_loc Path to your private key
  param: package String to be decrypted
  return decrypted string
  '''
  key = open(private_key_loc, "r").read()
  rsakey = RSA.importKey(key)
  decrypted = rsakey.decrypt(b64decode(package))
  return decrypted



# -*- coding: iso-8859-1 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the 
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
This module contains text related functions.

@author: Yann Cointepas
@organization: U{NeuroSpin<http://www.neurospin.org>} and U{IFR 49<http://www.ifr49.org>}
@license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''
__docformat__ = "epytext en"


def string_to_list( s ):
  s = s.strip( ' ' )
  while s:
    if s[0] == '"':
      i = 1
      while True:
        j = s[ i: ].find( '"' )
        if j == -1:
          # Syntax error, missing closing quote.
          # return the rest of the string and exit
          yield s[1:]
          return
        if s[ i + j - 1 ] == '\\':
          i = i + j + 1
        else:
          break
      yield s[ 1 : i+j ]
      s = s[ i+j+2: ].strip( ' ' )
    else:
      l = s.split( ' ', 1 )
      yield l[ 0 ]
      if len( l ) > 1:
        s = l[ 1 ].strip( ' ' )
      else:
        s = ''

def list_to_string( l ):
  return ' '.join( (quote_string( i ) for i in l ) )


def quote_string( unquoted ):
  if unquoted:
    if unquoted.find( ' ' ) > -1:
      return '"' + unquoted.replace( '"', '\\"' ) + '"'
    else:
      return unquoted
  return ''


def unquote_string( quoted ):
  if quoted and quoted[ 0 ] == '"':
    if quoted[ -1 ] == '"':
      quoted = quoted[ 1 : -1 ]
    else:
      # There is a syntax error here, quoted should
      # be terminated by a double quote
      quoted = quoted[ 1: ]
    return eval( 'r"' + quoted + '"' ).replace( '\\"', '"' )
  else:
    return quoted




# -*- coding: iso-8859-1 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the 
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.


'''
Base classes for writing various minf formats (XML, HDF5, Python's pickle, etc.)

@author: Yann Cointepas
@organization: U{NeuroSpin<http://www.neurospin.org>} and U{IFR 49<http://www.ifr49.org>}
@license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''

__docformat__ = "epytext en"

from soma.translation import translate as _


#------------------------------------------------------------------------------
class RegisterMinfWriterClass( type ):
  '''
  RegisterMinfWriterClass is used as metaclass of L{MinfWriter} to automatically
  register all classes derived from L{MinfWriter}.
  '''
  def __init__( cls, name, bases, dict ):
    if cls.name is not None:
      MinfWriter._allWriterClasses[ cls.name ] = cls


#------------------------------------------------------------------------------
class MinfWriter( object ):
  '''
  Class derived from MinfWriter are responsible of writing a specific format of
  minf file. This version only support XML format but other formats may 
  be added later (such as HDF5).
  All classes derived from L{MinfWriter} must define a L{name} class attribute
  the following methods:
    - L{__init__} to construct writer instances.
    - L{write} to write objects in minf file.
    - L{close} to terminate writing and close the min file.
  '''
  
  __metaclass__ = RegisterMinfWriterClass
  
  #: all classes derived from L{MinfWriter} are automatically stored in that
  #: dictionary (keys are formats name and values are class objects).
  _allWriterClasses = {}
  
  #: class derived from L{MinfWriter} must set a format name in this attribute.
  name = None
  
  
  def __init__( self, file, reducer ):
    '''
    Constructor of classes derived from L{MinfWriter} must be callable with two
    parameters.
    @param file: file object (opened for writing) where the minf file is 
      written.
    @type  file: any object respecting Python file object API
    @param reducer: name of the reducer to use (see L{soma.minf.tree} for 
      more information about reducers).
    @type  reducer: string
    '''
  
  
  def write( self, value ):
    '''
    Write an object into the minf file.
    @param value: any value that can be written in this minf file.
    '''
    
    
  def close( self ):
    '''
    Close the writer, further calls to L{write} method will lead to an error.
    '''
  
  
  def createWriter( destFile, format, reducer ):
    '''
    This static method create a L{MinfWriter} instance by looking for a 
    registered L{MinfWriter} derived classe named C{format}. Parameters
    C{destFile} and C{reducer} are passed to the derived class constructor. 
    @param format: name of the minf format.
    @type  format: string
    @returns: L{MinfWriter} derived class instance.
    @param file: file name or file object (opened for writing) where the minf
      file is written. If it is a file name, it is opened with
      C{open( destFile, 'wb' )}.
    @type  file: string or any object respecting Python file object API
    @param reducer: name of the reducer to use (see L{soma.minf.tree} for 
      more information about reducers).
    @type  reducer: string
    '''
    writer = MinfWriter._allWriterClasses.get( format )
    if writer is None:
      raise( _( 'No minf writer for format "%(format)s", possible formats are: %(possible)s' ) % \
             { 'format': format, 'possible': ', '.join( ['"' + i + '"' for i in MinfWriter._allWriterClasses ] ) } )
    if not hasattr( destFile, 'write' ):
      destFile = open( destFile, 'w' )
    return writer( destFile, reducer, )
  createWriter = staticmethod( createWriter )




# -*- coding: utf-8 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the 
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
Sorted dictionary behave like a dictionary but keep the item insertion
order.

@author: Yann Cointepas
@organization: U{NeuroSpin<http://www.neurospin.org>} and U{IFR 49<http://www.ifr49.org>}
@license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''
__docformat__ = "epytext en"

from UserDict import UserDict
from soma.undefined import Undefined

class SortedDictionary( UserDict, object ):
  '''
  Sorted dictionary behave like a dictionary but keep the item insertion
  order.
    
  Example::
    from SortedDictionary import SortedDictionary
    sd = SortedDictionary( ( 'fisrt', 1 ), ( 'second', 2 ) )
    sd[ 'third' ] = 3
    sd.insert( 0, 'zero', 0 )
    sd.items() == [('zero', 0), ('fisrt', 1), ('second', 2), ('third', 3)]
  '''
  def __init__( self, *args ):
    '''
    Initialize the dictionary with a list of ( key, value ) pairs.
    '''
    super(SortedDictionary, self).__init__()
    #UserDict.__init__(self)
    self.sortedKeys = []
    self.data = {}
    if len(args) == 1 and isinstance(args[0], list):
      elements = args[0] # dict / OrderedDict compatibility
    else:
      elements = args
    for key, value in elements:
      self[ key ] = value
      
  def keys( self ):
    '''
    @rtype: list
    @return: sorted list of keys
    '''
    return self.sortedKeys
  
  def items( self ):
    '''
    @rtype: list
    @return: sorted list of C{( key, value)} pairs
    '''
    return [x for x in self.iteritems()]

  def values( self ):
    '''
    @rtype: list
    @return: sorted list of values
    '''
    return [x for x in self.itervalues()]

  def __setitem__( self, key, value ):
    if not self.data.has_key( key ):
      self.sortedKeys.append( key )
    self.data[ key ] = value

  def __delitem__( self, key ):
    del self.data[ key ]
    self.sortedKeys.remove( key )

  def __getstate__( self ):
    return self.items()
    
  def __setstate__( self, state ):
    SortedDictionary.__init__( self, *state )

  def __iter__( self ):
    '''
    returns an iterator over the sorted keys
    '''
    return iter( self.sortedKeys )

  def iterkeys( self ):
    '''
    returns an iterator over the sorted keys
    '''
    return iter( self.sortedKeys )
  
  def itervalues( self ):
    '''
    returns an iterator over the sorted values
    '''
    for k in self:
      yield self[ k ]

  def iteritems( self ):
    '''
    returns an iterator over the sorted (key, value) pairs
    '''
    for k in self:
      try:
        yield ( k, self[ k ] )
      except KeyError:
        print '!SortedDictionary error!', self.data.keys(), self.sortedKeys
        raise

        
  def insert( self, index, key, value ):
    '''
    insert a ( C{key}, C{value} ) pair in sorted dictionary before position 
    C{index}. If C{key} is already in the dictionary, a C{KeyError} is raised.
    @type  index: integer
    @param index: index of C{key} in the sorted keys
    @param key: key to insert
    @param value: value associated to C{key}
    '''
    if self.data.has_key( key ):
      raise KeyError( key )
    self.sortedKeys.insert( index, key )
    self.data[ key ] = value

  def index(self, key):
   """
   Returns the index of the key in the sorted dictionary, or -1 if this key isn't in the dictionary.
   """ 
   try:
    i=self.sortedKeys.index(key)
   except:
    i=-1
   return i

  def clear( self ):
    '''
    Remove all items from dictionary
    '''
    del self.sortedKeys[:]
    self.data.clear()


  def sort(self, func=None):
    """Sorts the dictionary using function func to compare keys.

    @type func: function key*key->int
    @param func: comparison function, return -1 if e1<e2, 1 if e1>e2, 0 if e1==e2
    """
    self.sortedKeys.sort(func)


  def compValues(self, key1, key2):
    """
    Use this comparaison function in sort method parameter in order to sort the dictionary by values.
    if data[key1]<data[key2] return -1
    if data[key1]>data[key2] return 1
    if data[key1]==data[key2] return 0
    """
    e1=self.data[key1]
    e2=self.data[key2]
    print "comp", e1, e2
    if e1 < e2:
      return -1
    elif e1 > e2:
      return 1
    return 0
  
  
  def setdefault( self, key, value=None ):
    result = self.get( key, Undefined )
    if result is Undefined:
      self[ key ] = value
      result = value
    return result


  def pop( self, key, default=Undefined ):
    if default is Undefined:
      result = self.data.pop(key)
    else:
      result = self.data.pop(key,Undefined)
      if result is Undefined:
        return default
    self.sortedKeys.remove( key )
    return result


  def popitem( self ):
    result = self.data.popitem()
    try:
      self.sortedKeys.remove( result[0] )
    except ValueError:
      pass
    return result

  
  def __repr__( self ):
    return '{' + ', '.join( repr(k)+': '+repr(v) for k, v in self.iteritems() ) + '}'

    
class OrderedDict( SortedDictionary ):
  '''
  OrderedDict is fully compatible with Python 2.7 collections.OrderedDict.
  It is a SordedDictionary with a modified constructor API.
  '''
  def __init__( self, args=() ):
    super(OrderedDict, self).__init__(*args)



# -*- coding: utf-8 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the 
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
Utility functions for HTML format.

- author: Yann Cointepas
- organization: `NeuroSpin <http://www.neurospin.org>`_ and 
  `IFR 49 <http://www.ifr49.org>`_
- license: `CeCILL version 2 <http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>`_
'''
__docformat__ = 'restructuredtext en'

import htmlentitydefs

#------------------------------------------------------------------------------
#: mapping of charaters to be escaped for HTML
_htmlEscape = None
_lesserHtmlEscape = None

def htmlEscape( msg ):
  """
  Replace special characters in the message by their correponding html entity.
  
  - returns: *unicode*
  """
  global _htmlEscape
  if _htmlEscape is None:
    _htmlEscape=dict( [ (ord(j.decode('iso-8859-1')),u'&'+i+u';') for i,j in htmlentitydefs.entitydefs.iteritems() if len(j)==1] )
  return unicode( msg ).translate( _htmlEscape )

def lesserHtmlEscape( msg ):
  """
  Replace special characters in the message by their correponding html entity.
  
  - returns: *unicode*
  """
  global _lesserHtmlEscape
  if _lesserHtmlEscape is None:
    _lesserHtmlEscape=dict( [ (ord(j.decode('iso-8859-1')),u'&'+i+u';') for i,j in htmlentitydefs.entitydefs.iteritems() if len(j)==1 and j not in ( u'"', u'é', u'à', u'è', u'â', u'ê', u'ô', u'î', u'û', u'ù', u'ö', )] )
  return unicode( msg ).translate( _lesserHtmlEscape )



# -*- coding: iso-8859-1 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the 
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
:py:class:`BufferAndFile` instances are used to read data from a file (for 
instance for identification) and "put back" the data on the file. 

- author: Yann Cointepas
- organization: `NeuroSpin <http://www.neurospin.org>`_ and 
  `IFR 49 <http://www.ifr49.org>`_
- license: `CeCILL version 2 <http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>`_
'''
__docformat__ = 'restructuredtext en'

#------------------------------------------------------------------------------
class BufferAndFile( object ):
  '''
  This class is a read only file-like object that allows to read ahead and
  push data back into the stream. All pushed back data are stored in a buffer
  that is "read" by all subsequent read acces until it is empty. When the
  buffer is empty, reading is done directly on the attached file object.
  
  Example::
      
    from soma.bufferandfile import BufferAndFile
    
    # Open a file with a buffer
    f = BufferAndFile.open( fileName )
    # Check that the file content is XML
    start = f.read( 5 )
    # Put back the read characters
    f.unread( start )
    if start == '<?xml':
      # Use the file in an XML parser
      ...
    elif start == '&HDF':
      # Use the file in an HDF5 parser
      ...

  '''
  def __init__( self, file_object):
    '''
    Create a file-like object that adds an :py:meth:`unread` method to an 
    opened ``file_object``.
    '''
    super( BufferAndFile, self).__init__()
    self.__buffer = ''
    self.__file = file_object
    self.name = getattr( file_object, 'filename', 
                         getattr( file_object, 'name', '<unknown>' ) )
  
  
  def unread( self, string_value ):
    '''
    Adds data at the begining of the internal buffer. Data in the internal
    buffer will be returned by all subsequent read acces until the buffer is empty.
    '''
    self.__buffer = string_value + self.__buffer 
  
  
  def change_file( self, file_object ):
    '''
    Change the internal file object (keeps the internal buffer untouched).
    '''
    self.__file = file_object
  
  
  def clone( self ):
    '''
    Return a new L{BufferAndFile} instance with the same internale buffer and
    the same internal file object as C{self}.
    '''
    result = BufferAndFile( self.__file )
    result.__buffer = self.__buffer
    return result
  
  def read( self, size=None ):
    '''
    Read the file
    '''
    if size is None:
      result = self.__buffer + self.__file.read()
    else:
      buffer_size = len( self.__buffer )
      if buffer_size >= size:
        result = self.__buffer[ :size ]
        self.__buffer = self.__buffer[ size: ]
      else:
        result = self.__buffer + self.__file.read( size - buffer_size )
        self.__buffer = ''
    return result
  
  
  def readline( self, size=None ):
    '''
    Read one text line
    '''
    buffer_eol = self.__buffer.find( '\n' )
    if size is None:
      if buffer_eol < 0:
        result = self.__buffer + self.__file.readline()
        self.__buffer = ''
      else:
        buffer_eol += 1
        result = self.__buffer[ :buffer_eol ]
        self.__buffer = self.__buffer[ buffer_eol: ]
    else:
      if buffer_eol < 0:
        buffer_size = len( self.__buffer )
        if buffer_size >= size:
          result = self.__buffer[ :size ]
          self.__buffer = self.__buffer[ size: ]
        else:
          result = self.__buffer + self.__file.readline( size - buffer_size )
          self.__buffer = ''
      else:
        size = min( size, buffer_eol + 1 )
        result = self.__buffer[ :size ]
        self.__buffer = self.__buffer[ size: ]
    return result
  
  
  def __iter__( self ):
    '''
    Iteration protocol
    '''
    return self
  
  
  def next( self ):
    '''
    Iteration protocol
    '''
    line = self.readline()
    if not line:
      raise StopIteration
    return line


  def tell( self ):
    '''
    POsition in file
    '''
    return self.__file.tell() - len( self.__buffer )
  
  
  def seek( self, offset, whence=0 ):
    '''
    If ``whence`` is 0 or 2 (absolute seek positioning) or if offset is
    negative, internal buffer is cleared and seek is done directly on the
    internal file object. Otherwise (relative seek with a positive offset),
    internal buffer is taken into account.
    '''
    if whence == 2 or whence == 0 or offset < 0:
      self.__buffer = ''
      return self.__file.seek( offset, whence )
    else:
      buflen = len( self.__buffer )
      if offset > buflen:
        self.__buffer = ''
        return self.__file.seek( offset - buflen, whence )
      else:
        self.__buffer = self.__buffer[ offset: ]
  
  
  def open( *args, **kwargs ):
    '''
    Open a file with built-in :py:func:`python:open` and create a 
    :py:class:`BufferAndFile` instance.
    '''
    return BufferAndFile( open( *args, **kwargs ) )
  open = staticmethod( open )



# -*- coding: iso-8859-1 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the 
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
Singleton pattern.

- author: Yann Cointepas
- organization: `NeuroSpin <http://www.neurospin.org>`_ and 
  `IFR 49 <http://www.ifr49.org>`_
- license: `CeCILL version 2 <http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>`_
'''
from __future__ import absolute_import
__docformat__ = 'restructuredtext en'


class Singleton( object ):
  '''
  Implements the singleton pattern. A class deriving from ``Singleton`` can
  have only one instance. The first instanciation will create an object and
  other instanciations return the same object. Note that the :py:meth:`__init__`
  method (if any) is still called at each instanciation (on the same object).
  Therefore, :py:class:`Singleton` derived classes should define 
  :py:meth:`__singleton_init__`
  instead of :py:meth:`__init__` because the former is only called once.
  
  Example::

    from singleton import Singleton
    
    class MyClass( Singleton ):
      def __singleton_init__( self ):
        self.attribute = 'value'
    
    o1 = MyClass()
    o2 = MyClass()
    print o1 is o2
  
  '''
  def __new__( cls, *args, **kwargs ):
    if '_singleton_instance' not in cls.__dict__:
      cls._singleton_instance = super(Singleton, cls).__new__( cls )
      singleton_init = getattr( cls._singleton_instance, 
                               '__singleton_init__', None )
      if singleton_init is not None:
        singleton_init( *args, **kwargs )
    return cls._singleton_instance

  def __init__( self, *args, **kwargs ):
    '''
    The __init__ method of :py:class:`Singleton` derived class should do 
    nothing. 
    Derived classes must define :py:meth:`__singleton_init__` instead of __init__.
    '''

  def __singleton_init__( self, *args, **kwargs ):
    super(Singleton, self).__init__( *args, **kwargs )




# -*- coding: utf-8 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the 
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
Some useful functions to manage file or directorie names.

author: Yann Cointepas
organization: U{NeuroSpin<http://www.neurospin.org>} and U{IFR 49<http://www.ifr49.org>}
license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''
__docformat__ = "restructuredtext en"

import os
import platform
import fnmatch
import hashlib

def split_path( path ):
  '''
  Iteratively apply C{os.path.split} to build a list. Ignore trailing directory separator.

  Examples::
    split_path( '/home/myaccount/data/file.csv' ) returns [ '/', 'home', 'myaccount', 'data', 'file.csv' ]
    split_path( 'home/myaccount/data/file.csv' ) returns [ 'home', 'myaccount', 'data', 'file.csv' ]
    split_path( '/home/myaccount/data/' ) returns [ '/', 'home', 'myaccount', 'data' ]
    split_path( '/home/myaccount/data' ) returns [ '/', 'home', 'myaccount', 'data' ]
    split_path( '' ) returns [ '' ]
'''
  result = []
  a,b = os.path.split( path )
  if not b:
    a,b = os.path.split( a )
  while a and b:
    result.insert( 0, b )
    a,b = os.path.split( a )
  if a:
    result.insert( 0, a )
  else:
    result.insert( 0, b )
  return result


def relative_path( path, referenceDirectory ):
  '''
  Return a relative version of a path given a
  reference directory.
  
  os.path.join( referenceDirectory, relative_path( path, referenceDirectory ) )
  returns os.path.abspath( path )
  
  Example
  =======
    relative_path( '/usr/local/brainvisa-3.1/bin/brainvisa', '/usr/local' )
    returns 'brainvisa-3.1/bin/brainvisa'
    
    relative_path( '/usr/local/brainvisa-3.1/bin/brainvisa', '/usr/local/bin' )
    returns '../brainvisa-3.1/bin/brainvisa'
    
    relative_path( '/usr/local/brainvisa-3.1/bin/brainvisa', '/tmp/test/brainvisa' )
    returns '../../../usr/local/brainvisa-3.1/bin/brainvisa'
  '''
  sPath = split_path( os.path.abspath( path ) )
  sReferencePath = split_path( os.path.abspath( referenceDirectory ) )
  i = 0
  while i < len( sPath ) and i < len( sReferencePath ) and sPath[ i ] == sReferencePath[ i ]:
    i += 1
  plist = ( [ '..' ] * ( len( sReferencePath ) - i  ) ) + sPath[ i: ]
  if len( plist ) == 0:
    return ''
  return os.path.join( *plist )


def no_symlink( path ):
  '''
  Read all symlinks in path to return the "real" path.
  
  Example
  =======
    With the following configuration::
      /usr/local/software-1.0 is a directory
      /usr/local/software-1.0/bin is a directory
      /usr/local/software-1.0/bin/command is a file
      /usr/local/software is a symlink to software-1.0
      /home/bin/command is a symlink to /usr/local/software/bin/command
    no_symlink( '/home/bin/command' ) would return '/usr/local/software-1.0/bin/command' 
   
  '''
  s = split_path( p )
  p=''
  while s:
    p = os.path.join( p, s.pop( 0 ) )
    while os.path.islink(p):
      d,f = os.path.split(p)
      p = os.path.normpath( os.path.join( d, os.readlink( p ) ) )
  return p


#: Character used to separate directories in environment variables such as PATH
path_separator = os.pathsep


def find_in_path( file, path = None ):
  '''
  Look for a file in a series of directories. By default, directories are
  contained in C{PATH} environment variable. But another environment variable
  name or a sequence of directories names can be given in C{path} parameter.
  
  Examples::
    find_in_path( 'sh' ) could return '/bin/sh'
    find_in_path( 'libpython2.5.so', 'LD_LIBRARY_PATH' ) could return '/usr/local/lib/libpython2.5.so'
  '''
  if path is None:
    path = os.environ.get( 'PATH' ).split( path_separator )
  elif isinstance( path, basestring ):
    var = os.environ.get( path )
    if var is None:
      var = path
    path = var.split( path_separator )
  for i in path:
    p = os.path.normpath( os.path.abspath( i ) )
    if p:
      r = os.path.join( p, file )
      if os.path.isdir( p ) and os.path.exists( r ):
        return r

def locate_file( pattern, root = os.curdir ):
    """
    Locates a file in a directory tree
    
    :param string pattern:
        The pattern to find
    
    :param string root:
        The search root directory
    
    :returns:
        The first found occurrence
    """
    for path, dirs, files in os.walk(os.path.abspath(root)):
        for filename in fnmatch.filter(files, pattern):
            return os.path.join(path, filename)

def which(program):
    """
    Identifies the location of an executable
    
    :param string program
        The executable to find
        
    :returns:
        The full path of the executable
    """
    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)
    
    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            path = path.strip('"')
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file
    
    return None

def update_hash_from_directory(directory, hash):
    '''
    Update a hash object from the content of a directory. The hash will
    reflect the recursive content of all files as well as the paths in all 
    directories.
    '''
    for root, dirs, files in sorted(os.walk(directory)):
        for file in sorted(files):
            hash.update(file)
            hash.update(open(os.path.join(root, file)).read())
        for dir in sorted(dirs):
            hash.update(dir)
            update_hash_from_directory(os.path.join(root,dir), hash)

def path_hash(path, hash=None):
    '''
    Return a hash hexdigest for a file or a directory.
    '''
    if hash is None:
        hash = hashlib.md5()
    if os.path.isdir(path):
        update_hash_from_directory(path, hash)
    else:
        hash.update(open(path).read())
    return hash.hexdigest()


def ensure_is_dir(d, clear_dir=False):
    """ If the directory doesn't exist, use os.makedirs """
    if not os.path.exists(d):
        os.makedirs(d)
    elif clear_dir:
        shutil.rmtree(d)
        os.makedirs(d)





# -*- coding: utf-8 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the 
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
This module contains the L{XHTML} class that contains an XHTML tree that
can be saved in minf files.

@author: Yann Cointepas
@author: Someone Else
@organization: U{NeuroSpin<http://www.neurospin.org>} and U{IFR 49<http://www.ifr49.org>}
@license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''
__docformat__ = "epytext en"

import types
from cStringIO import StringIO

from soma.translation import translate as _
from soma.minf.api import readMinf
from soma.minf.xml_tags import minfTag, expanderAttribute, xhtmlTag
from soma.html import lesserHtmlEscape
from xml.sax.saxutils import quoteattr as xml_quoteattr

#------------------------------------------------------------------------------
class XHTML:
  '''
  Instances of L{XHTML} contains the structure of an XHTML tree and can be used
  to produce an XML string or an HTML string.
  '''
  
  
  def __init__( self, tag, attributes=None, content=None ):
    '''
    Construct an XHTML tree composed of a tag name, a dictionary containing
    attributes and a content composed of a series of strings and/or XHTML
    values.
    '''
    if attributes is None: attributes = {}
    if content is None: content = []
    self.tag = tag
    self.attributes = attributes
    self.content = content
  
  
  def __getinitkwargs__( self ):
    d= {}
    if self.attributes: d[ 'attributes' ] = self.attributes
    if self.content: d[ 'content' ] = self.content
    return ( self.tag, ), d
  
  def _contentXML( content ):
    return ''.join( [ XHTML._itemXML( i ) for i in content ] )
  _contentXML = staticmethod( _contentXML )
  
  
  def _itemXML( item ):
    if isinstance( item, XHTML ):
      if item.tag is None:
        # When tag is None, no tag info is produced
        # This is useful for creating XHTML converters
        # that can remove tags.
        return item._contentXML( item.content )
      else:
        result = '<' + unicode( item.tag )
        if item.attributes:
          result += ' ' + ' '.join( 
            [ unicode( a ) + '="' + unicode( v  )+ '"' \
              for a, v in item.attributes.iteritems() ] )
        if item.content:
          result += '>' + item._contentXML( item.content ) + '</' + \
                    str( item.tag ) + '>'
        else:
          result += '/>'
        return result
    else:
      return lesserHtmlEscape( item )
  _itemXML = staticmethod( _itemXML )

  
  def __minfxml__( self, xmlWriter, attributes, level ):
    backupAttributes = self.attributes
    self.attributes = self.attributes.copy()
    self.attributes.update( attributes )
    xmlWriter._encodeAndWriteLine( self._itemXML( self ), level )
    self.attributes = backupAttributes
  
  
  def buildFromHTML( html ):
    '''
    Return an XHTML instance build from an HTML string.
    @param html: piece of an HTML file
    @type  html: unicode
    '''
    # html must be an unicode string. if not, we can fail to decode it because we don't know what encoding has been use to encode it.
    #try:
      #if not isinstance( html, unicode ):
        #html = unicode( html) #, 'iso-8859-1' )
    #except: # decoding of the string can fail because it isn't encoded with the default charset
        #pass
    io = StringIO()
    # when unicode string is written in a stream, default encoding is used to encode it : 
    io.write( \
      '<?xml version="1.0" encoding="utf-8" ?>\n<' + minfTag + ' ' + \
      expanderAttribute + '="minf_2.0">\n<' + xhtmlTag + '>' + \
      html.encode("utf-8") + '</' + xhtmlTag + '>\n</' + minfTag + '>' )
    io.seek( 0 )
    return readMinf( io )[ 0 ]
  buildFromHTML = staticmethod( buildFromHTML )


  def xml( item ):
    '''
    Build an XML unicode string based on either an XML string (in that case it
    is returned as is) or an XHTML instance.
    @param item: value to convert in XML.
    @type  item: XHTML instance or unicode containing XML
    '''
    if isinstance( item, types.StringTypes ):
      return item
    elif isinstance( item, XHTML ):
      return item._itemXML( item )
    else:
      raise RuntimeError( _( 'Cannot use XHTML converter for %s' ) % ( str(item),) )
  xml = staticmethod( xml )

  
  def html( item ):
    '''
    Build an HTML unicode string based on either an HTML string (in that case it
    is returned as is) or an XHTML instance.
    @param item: value to convert in HTML.
    @type  item: XHTML instance or unicode containing HTML
    '''
    if isinstance( item, types.StringTypes ):
      return item
    elif isinstance( item, XHTML ):
      return item._contentXML( item.content )
    else:
      raise RuntimeError( _( 'Cannot use XHTML converter for %s' ) % ( unicode(item),) )
  html = staticmethod( html )



# -*- coding: utf-8 -*-
import sipconfig
from soma.qt_gui.qt_backend import QtGui, QtCore
from soma.controller import trait_ids, Controller
from soma.utils.functiontools import partial, SomaPartial
from soma.qt4gui.api import TimeredQLineEdit
from soma.global_naming import GlobalNaming
from soma.gui.icon_factory import IconFactory
try:
    from traits.api import File, Directory
except ImportError:
    from enthought.traits.api import File, Directory
import weakref


#-------------------------------------------------------------------------
class StrCreateWidget(object):

    @staticmethod
    def is_valid_trait(trait):
        return True

    @staticmethod
    def create_widget(parent, name, trait, value):
        attribute_widget = TimeredQLineEdit(parent)
        label = trait.label
        if not label:
            label = name
        if label is not None:
            label_widget = QtGui.QLabel(label, parent)
        else:
            label_widget = None

        return (attribute_widget, label_widget)

    @staticmethod
    def update_controller(controller_widget, name, attribute_widget):
        """GUI modified so update traits"""
        if isinstance(attribute_widget, TimeredQLineEdit):
            try:
                setattr(
                    controller_widget.controller,
                    name,
                    unicode(attribute_widget.text()))
            except:
                print 'Warning - StrCreateWidget.update_controller: controller widget set failure'
        else:
            setattr(
                controller_widget.controller,
                name,
                unicode(attribute_widget.text_widget.text()))

    @staticmethod
    def update_controller_widget(
            controller_widget, name, attribute_widget, label_widget):
        """Traits modified so update GUI"""
        if isinstance(attribute_widget, TimeredQLineEdit):
            attribute_widget.setText(
                unicode(getattr(controller_widget.controller, name, '')))
        else:
            attribute_widget.text_widget.setText(
                unicode(getattr(controller_widget.controller, name, '')))

    @classmethod
    def connect_controller(
            cls, controller_widget, name, attribute_widget, label_widget):
        widget_hook = partial(cls.update_controller,
                              weakref.proxy(controller_widget), name,
                              attribute_widget)
        if isinstance(attribute_widget, TimeredQLineEdit):
            attribute_widget.userModification.connect(widget_hook)
        else:
            attribute_widget.text_widget.userModification.connect(widget_hook)

        controller_hook = SomaPartial(cls.update_controller_widget,
                                      weakref.proxy(controller_widget), name,
                                      attribute_widget,
                                      label_widget)
        controller_widget.controller.on_trait_change(
            controller_hook, name=name)
        attribute_widget._controller_connections = (
            widget_hook, controller_hook)

    @staticmethod
    def disconnect_controller(
            controller_widget, name, attribute_widget, label_widget):
        widget_hook, controller_hook = attribute_widget._controller_connections
        controller_widget.controller.on_trait_change(
            controller_hook, name=name, remove=True)
        if isinstance(attribute_widget, TimeredQLineEdit):
            attribute_widget.userModification.disconnect(widget_hook)
        else:
            attribute_widget.text_widget.userModification.disconnect(
                widget_hook)
        del attribute_widget._controller_connections


#-------------------------------------------------------------------------
class FloatCreateWidget(StrCreateWidget):

    @staticmethod
    def update_controller(controller_widget, name, attribute_widget):
        setattr(
            controller_widget.controller,
            name,
            float(attribute_widget.text()))

    @staticmethod
    def update_controller_widget(
            controller_widget, name, attribute_widget, label_widget):
        attribute_widget.setText(
            unicode(getattr(controller_widget.controller, name, 0)))


#-------------------------------------------------------------------------
class IntCreateWidget(FloatCreateWidget):

    @staticmethod
    def update_controller(controller_widget, name, attribute_widget):
        setattr(
            controller_widget.controller,
            name,
            int(attribute_widget.text()))


#-------------------------------------------------------------------------
class LongCreateWidget(FloatCreateWidget):

    @staticmethod
    def update_controller(controller_widget, name, attribute_widget):
        setattr(
            controller_widget.controller,
            name,
            long(attribute_widget.text()))


#-------------------------------------------------------------------------
class BoolCreateWidget(object):

    @staticmethod
    def is_valid_trait(trait):
        return True

    @staticmethod
    def create_widget(parent, name, trait, value):
        attribute_widget = QtGui.QCheckBox(parent)
        label = trait.label
        if not label:
            label = name
        if label is not None:
            label_widget = QtGui.QLabel(label, parent)
        else:
            label_widget = None
        return (attribute_widget, label_widget)

    @staticmethod
    def update_controller(controller_widget, name, attribute_widget):
        setattr(
            controller_widget.controller,
            name,
            bool(attribute_widget.isChecked()))

    @staticmethod
    def update_controller_widget(
            controller_widget, name, attribute_widget, label_widget):
        attribute_widget.setChecked(
            getattr(
                controller_widget.controller,
                name,
                False))

    @classmethod
    def connect_controller(
            cls, controller_widget, name, attribute_widget, label_widget):
        widget_hook = partial(cls.update_controller,
                              weakref.proxy(controller_widget), name,
                              attribute_widget)
        attribute_widget.clicked.connect(widget_hook)
        controller_hook = SomaPartial(cls.update_controller_widget,
                                      weakref.proxy(controller_widget), name,
                                      attribute_widget,
                                      label_widget)
        controller_widget.controller.on_trait_change(
            controller_hook, name=name)
        attribute_widget._controller_connections = (
            widget_hook, controller_hook)

    @staticmethod
    def disconnect_controller(
            controller_widget, name, attribute_widget, label_widget):
        widget_hook, controller_hook = attribute_widget._controller_connections
        controller_widget.controller.on_trait_change(
            controller_hook, name=name, remove=True)
        attribute_widget.clicked.disconnect(widget_hook)
        del attribute_widget._controller_connections


#-------------------------------------------------------------------------
class EnumCreateWidget(object):

    @staticmethod
    def is_valid_trait(trait):
        return True

    @staticmethod
    def create_widget(parent, name, trait, value):
        attribute_widget = QtGui.QComboBox(parent)
        values = trait.handler.values
        labels = getattr(trait, 'labels')
        if not labels:
            labels = ()
        labels += (None, ) * (len(values) - len(labels))
        labels = [(unicode(value) if label is None else label)
                  for value, label in zip(values, labels)]
        attribute_widget._values = values
        for label in labels:
            attribute_widget.addItem(label)
        label = trait.label
        if not label:
            label = name
        if label is not None:
            label_widget = QtGui.QLabel(label, parent)
        else:
            label_widget = None
        return (attribute_widget, label_widget)

    @staticmethod
    def update_controller(controller_widget, name, attribute_widget, value):
        # default_value_change(getattr(controller_widget.controller,name),attribute_widget._values[
        # attribute_widget.currentIndex() ],attribute_widget.btn_default_value)
        setattr(
            controller_widget.controller,
            name,
            attribute_widget._values[attribute_widget.currentIndex()])

    @staticmethod
    def update_controller_widget(
            controller_widget, name, attribute_widget, label_widget):
        label = getattr(controller_widget.controller, name, None)
        if label is not None:
            attribute_widget.setCurrentIndex(
                attribute_widget._values.index(label))

    @classmethod
    def connect_controller(
            cls, controller_widget, name, attribute_widget, label_widget):
        widget_hook = partial(cls.update_controller,
                              weakref.proxy(controller_widget), name,
                              attribute_widget)
        attribute_widget.activated.connect(widget_hook)
        controller_hook = SomaPartial(cls.update_controller_widget,
                                      weakref.proxy(controller_widget), name,
                                      attribute_widget,
                                      label_widget)
        controller_widget.controller.on_trait_change(
            controller_hook, name=name)
        attribute_widget._controller_connections = (
            widget_hook, controller_hook)

    @staticmethod
    def disconnect_controller(
            controller_widget, name, attribute_widget, label_widget):
        widget_hook, controller_hook = attribute_widget._controller_connections
        controller_widget.controller.on_trait_change(
            controller_hook, name=name, remove=True)
        attribute_widget.activated.disconnect(widget_hook)
        del attribute_widget._controller_connections


#-------------------------------------------------------------------------
class StrEnumCreateWidget(object):

    @staticmethod
    def is_valid_trait(trait):
        return trait.values is not None

    @staticmethod
    def create_widget(parent, name, trait, value):
        values = list(trait.values)
        if trait.labels:
            raise ValueError('labels is not allowed for Str with values')
        attribute_widget = QtGui.QComboBox(parent)
        attribute_widget.setEditable(True)
        attribute_widget._values = values
        for label in values:
            attribute_widget.addItem(label)
        label = trait.label
        if not label:
            label = name
        if label is not None:
            label_widget = QtGui.QLabel(label, parent)
        else:
            label_widget = None
        return (attribute_widget, label_widget)

    @staticmethod
    def update_controller(controller_widget, name, attribute_widget, value):
        setattr(
            controller_widget.controller,
            name,
            unicode(attribute_widget.currentText()))

    @staticmethod
    def update_controller_widget(
            controller_widget, name, attribute_widget, label_widget):
        value = getattr(controller_widget.controller, name, '')
        try:
            index = attribute_widget._values.index(value)
        except ValueError:
            index = -1
        if index != -1:
            attribute_widget.setCurrentIndex(index)
        else:
            attribute_widget.setEditText(value)

    @classmethod
    def connect_controller(
            cls, controller_widget, name, attribute_widget, label_widget):
        widget_hook = partial(cls.update_controller,
                              weakref.proxy(controller_widget), name,
                              attribute_widget)
        attribute_widget.editTextChanged.connect(widget_hook)
        controller_hook = SomaPartial(cls.update_controller_widget,
                                      weakref.proxy(controller_widget), name,
                                      attribute_widget,
                                      label_widget)
        controller_widget.controller.on_trait_change(
            controller_hook, name=name)
        attribute_widget._controller_connections = (
            widget_hook, controller_hook)

    @staticmethod
    def disconnect_controller(
            controller_widget, name, attribute_widget, label_widget):
        widget_hook, controller_hook = attribute_widget._controller_connections
        controller_widget.controller.on_trait_change(
            controller_hook, name=name, remove=True)
        attribute_widget.editTextChanged.disconnect(widget_hook)
        del attribute_widget._controller_connections


#-------------------------------------------------------------------------
class FileCreateWidget(object):

    @staticmethod
    def is_valid_trait(trait):
        return True

    @classmethod
    def create_widget(cls, parent, name, trait, value):
        text_widget, label_widget = StrCreateWidget.create_widget(
            parent, name, trait, value)
        attribute_widget = QtGui.QWidget(parent)
        horizontal_layout = QtGui.QHBoxLayout(attribute_widget)
        horizontal_layout.setContentsMargins(0, 0, 0, 0)
        horizontal_layout.addWidget(text_widget)
        attribute_widget.text_widget = text_widget

        btn_browse = QtGui.QPushButton(attribute_widget)
        if trait.output is True:
            btn_browse.setIcon(QtGui.QIcon(IconFactory()._imageBrowseOutput))
        if trait.output is False:
            btn_browse.setIcon(QtGui.QIcon(IconFactory()._imageBrowseInput))
        if trait(name).is_trait_type(Directory):
            btn_browse.setIcon(QtGui.QIcon(IconFactory()._imageBrowseDir))
        horizontal_layout.addWidget(btn_browse)
        btn_browse.setFixedSize(20, 20)
        attribute_widget.btn_browse = btn_browse
        attribute_widget._browse_hook = partial(
            cls.file_dialog, weakref.proxy(parent), name,
            weakref.proxy(attribute_widget))
        attribute_widget.btn_browse.clicked.connect(
            attribute_widget._browse_hook)

        viewers = getattr(parent.controller, 'viewers', None)
        if viewers and name in viewers:
            btn_viewer = QtGui.QPushButton(attribute_widget)
            horizontal_layout.addWidget(btn_viewer)
            btn_viewer.setFixedSize(20, 20)
            btn_viewer.setIcon(QtGui.QIcon(IconFactory()._imageViewer))
            attribute_widget.btn_viewer = btn_viewer
            if hasattr(parent.controller, 'call_viewer'):
                viewer_hook = partial(
                    parent.controller.call_viewer,
                    parent,
                    name)
                attribute_widget._viewer_hook = viewer_hook
                attribute_widget.btn_viewer.clicked.connect(viewer_hook)
        return (attribute_widget, label_widget)

    @staticmethod
    def file_dialog(parent, name, attribute_widget):
        if hasattr( parent.controller.user_traits()[name], 'output' ) \
                and parent.controller.user_traits()[name].output:
            outputfile = True
        else:
            outputfile = False
        if sipconfig.Configuration().sip_version >= 0x040a00:
            #/nfs/neurospin/cati/cati_shared/MEMENTO/CONVERTED/001/0010020_LAFR/M000/MRI/3DT1/0010020_LAFR_M000_3DT1_S002.nii.gz
                #''
            if not outputfile:
                value = QtGui.QFileDialog.getOpenFileName(
                    parent, 'Select a file', '', '',
                    options=QtGui.QFileDialog.DontUseNativeDialog)
            else:
                value = QtGui.QFileDialog.getSaveFileName(
                    parent, 'Select a file', '', '',
                    options=QtGui.QFileDialog.DontUseNativeDialog)
        else:
            if outputfile:
                value = QtGui.QFileDialog.getOpenFileName(
                    self._widget, 'Select a file', '', '',
                    0, QtGui.QFileDialog.DontUseNativeDialog)
            else:
                value = QtGui.QFileDialog.getSaveFileName(
                    self._widget, 'Select a file', '', '',
                    0, QtGui.QFileDialog.DontUseNativeDialog)
        setattr(parent.controller, name, unicode(value))

    @staticmethod
    def update_viewer(controller_widget, name, attribute_widget):
        if hasattr(controller_widget.controller, 'viewers'):
            if name in controller_widget.controller.viewers:
                try:
                    open(getattr(controller_widget.controller, name))
                    attribute_widget.btn_viewer.setEnabled(True)
                except IOError:
                    attribute_widget.btn_viewer.setEnabled(False)

    @staticmethod
    def update_controller(controller_widget, name, attribute_widget):
        """GUI modified so update traits"""
        StrCreateWidget.update_controller(
            controller_widget,
            name,
            attribute_widget.text_widget)
        # StrCreateWidget.update_controller( controller_widget, name,
        # attribute_widget)

    @staticmethod
    def update_controller_widget(
            controller_widget, name, attribute_widget, label_widget):
        """Traits modified so update GUI"""
        StrCreateWidget.update_controller_widget(
            controller_widget,
            name,
            attribute_widget.text_widget,
            label_widget)
        # StrCreateWidget.update_controller_widget( controller_widget, name,
        # attribute_widget, label_widget )

        # To disable viewer if file doesn't exist
        # if controller_widget.controller.trait( name ).viewer is not None:
        if hasattr(controller_widget.controller, 'viewers'):
            if name in controller_widget.controller.viewers:
                try:
                    open(getattr(controller_widget.controller, name))
                    attribute_widget.btn_viewer.setEnabled(True)
                except IOError:
                    attribute_widget.btn_viewer.setEnabled(False)

    @classmethod
    def connect_controller(
            cls, controller_widget, name, attribute_widget, label_widget):
        # StrCreateWidget.connect_controller( controller_widget, name,
        # attribute_widget.text_widget, label_widget )
        StrCreateWidget.connect_controller(
            controller_widget,
            name,
            attribute_widget.text_widget,
            label_widget)

        # Function call when traits change to update if viewer enabled or not
        controller_hook = SomaPartial(cls.update_viewer,
                                      weakref.proxy(controller_widget), name,
                                      attribute_widget)
        controller_widget.controller.on_trait_change(controller_hook, name)

        attribute_widget._controller_connections = controller_hook

    @staticmethod
    def disconnect_controller(
            controller_widget, name, attribute_widget, label_widget):
        controller_hook = attribute_widget._controller_connections
        controller_widget.controller.on_trait_change(
            controller_hook, name, remove=True)
        StrCreateWidget.disconnect_controller(
            controller_widget,
            name,
            attribute_widget.text_widget,
            label_widget)
        del attribute_widget._controller_connections


#-------------------------------------------------------------------------
class DirectoryCreateWidget(FileCreateWidget):

    @staticmethod
    def file_dialog(controller_widget, name, attribute_widget):
        value = QtGui.QFileDialog.getExistingDirectory(
            controller_widget, 'Select a directory', '',
            options=QtGui.QFileDialog.ShowDirsOnly | QtGui.QFileDialog.DontUseNativeDialog)
        setattr(controller_widget.controller, name, unicode(value))


def find_function_viewer(name_viewer):
    return GlobalNaming().get_object(name_viewer)


# def default_value_change(old,new,btn):
        # if old !=new:
                # btn.setVisible(True)



# -*- coding: utf-8 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the 
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
This module contains all the framework necessary to customize, read and
write minf files. A minf file is composed of structured data saved in
XML or Python format. The minf framework provides tools to read and write
minf files but also to customize the way Python objects are read an written.

There are several submodules in this package but main functions and classes
can be imported from C{soma.minf.api}:
  - for reading minf files: L{iterateMinf}, L{readMinf}
  - for writing minf files: L{createMinfWriter}, L{writeMinf}
  - for customizing minf files: L{createReducerAndExpander}, L{registerClass}, L{registerClassAs}

@author: Yann Cointepas
@organization: U{NeuroSpin<http://www.neurospin.org>} and U{IFR 49<http://www.ifr49.org>}
@license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''
__docformat__ = "restructuredtext en"


import gzip


from soma.translation import translate as _
from soma.minf.error import MinfError
from soma.bufferandfile import BufferAndFile
from soma.minf.reader import MinfReader
from soma.minf.writer import MinfWriter
from soma.minf.tree import createReducerAndExpander, registerClass, \
                            registerClassAs, createMinfExpander, \
                            EndStructure, MinfReducer, MinfExpander, \
                            listStructure, dictStructure
from soma.undefined import Undefined
defaultReducer = MinfReducer.defaultReducer


#------------------------------------------------------------------------------
def minfFormat( source ):
  '''
  Return a pair C{( format, reduction )} identifying the minf format. If
  C{source} is not a minf file, C{( None, None )} is returned. Otherwise,
  C{format} is a string representing the format of the minf file: C{'XML'} or
  C{'python'}. C{reduction} is the name of the reducer used to write the minf
  file or C{None} if format is C{'python'}.

  Example::
    from soma.minf.api import minfFormat
    format, reduction = minfFormat( '/home/me/test.minf' )

  If C{source} is a L{BufferAndFile} instance, this call behave as if nothing
  has been read from the file. This can be useful if you have an opened file
  that cannot be seeked backward:

  Example::
    from soma.bufferandfile import BufferAndFile
    from soma.minf.api import minfFormat, readMinf

    bf = BufferAndFile( stream_file_object )
    format, reduction = minfFormat( bf )
    if format is not None:
      minfContent = readMinf( bf )


  @param source: Input file name or file object. If it is a file name, it is
    opened with C{open( source )}.
  '''
  if not hasattr( source, 'readline' ):
    source = BufferAndFile( open( source ) )
  elif not isinstance( source, BufferAndFile ):
    source.seek(0)
    source = BufferAndFile( source )

  # Check first non white character to see if the minf file is XML or not
  start = source.read( 5 )
  if start == 'attri':
    source.unread( start )
    return ( 'python', None )
  elif start != '<?xml':
    # Try gzip compressed file
    gzipSource = source.clone()
    gzipSource.unread( start )
    gunzipSource = gzip.GzipFile( source.name ) 
    try:
      start = gunzipSource.read( 5 )
    except IOError:
      start = ''
    if start != '<?xml':
      raise MinfError( _( 'Invalid minf file: %s' ) % ( source.name, ) )
    source.change_file( gunzipSource )
    source.unread( start )
  else:
    source.unread( start )

  r = MinfReader.createReader( 'XML' )
  reduction, buffer = r.reduction( source )
  source.unread( buffer )
  return( 'XML', reduction )


#------------------------------------------------------------------------------
def _setTarget( target, source ):
  from soma.signature.api import HasSignature
  if isinstance( source, dict ):
    if isinstance( target, dict ):
      target.update( source )
      return True
    elif isinstance( target, HasSignature ):
      for k, v in source.iteritems():
        attrTarget = getattr( target, k, Undefined )
        if attrTarget is Undefined:
          setattr( target, k, v )
        else:
          if not _setTarget( attrTarget, v ):
            setattr( target, k, v )
      return True
  return False
  
  
#------------------------------------------------------------------------------
def iterateMinf( source, targets=None, stop_on_error=True, exceptions=[] ):
  '''
  Returns an iterator over all objects stored in a minf file.

  Example::
    from soma.minf.api import iterateMinf

    for item in iterateMinf( 'test.minf' ):
      print repr( item )

  @param source: Input file name or file object. If it is a file name, it is
    opened with C{open( source )}.
  '''
  if targets is not None:
    targets = iter( targets )
  if not hasattr( source, 'readline' ):
    source = BufferAndFile( open( source ) )
  elif not isinstance( source, BufferAndFile ):
    source.seek(0)
    source = BufferAndFile( source )

  # Check first non white character to see if the minf file is XML or not
  start = source.read( 5 )
  source.unread( start )
  if start == 'attri':
    try:
      import numpy
      d = { 'nan' : numpy.nan }
    except:
      d = { 'nan' : None }
    try:
      exec source.read().replace("\r\n", "\n") in d
    except Exception, e:
      x = source
      if hasattr( source, '_BufferAndFile__file' ):
        x = source._BufferAndFile__file
      x = 'Error in iterateMinf while reading ' + str(x) + ': '
      msg = x + e.message
      # e.message = msg
      # e.args = ( x + e.args[0], ) + e.args[1:] 
      print x
      raise
    minf = d[ 'attributes' ]
    if targets is not None:
      result = targets.next()
      _setTarget( result, minf )
      yield result
    else:
      yield minf
    return
  elif start != '<?xml':
    # Try gzip compressed file
    gzSource = gzip.GzipFile( source.name )
    if gzSource.read( 5 ) != '<?xml':
      raise MinfError( _( 'Invalid minf file: %s' ) % ( source.name, ) )
    source = BufferAndFile( gzSource )
    source.unread( '<?xml' )

  r = MinfReader.createReader( 'XML' )
  iterator = r.nodeIterator( source )
  minfNode = iterator.next()
  expander = createMinfExpander( minfNode.attributes[ 'reduction' ] )
  count = 0
  for nodeItem in iterator:
    count += 1
    if isinstance( nodeItem, EndStructure ):
      break
    target = None
    if targets is not None:
      try:
        target = targets.next()
      except StopIteration:
        targets = None
    yield expander.expand( iterator, nodeItem, target=target, stop_on_error=stop_on_error, exceptions=exceptions )

#------------------------------------------------------------------------------
def readMinf( source, targets=None, stop_on_error=True, exceptions=[] ):
  '''
  Entirerly reads a minf file and returns its content in a tuple.
  Equivalent to C{tuple( iterateMinf( source ) )}.
  @see: L{iterateMinf}
  '''
  return tuple( iterateMinf( source, targets=targets, stop_on_error=stop_on_error, exceptions=exceptions ) )


#------------------------------------------------------------------------------
def createMinfWriter( destFile, format='XML', reducer='minf_2.0' ):
  '''
  Create a writer for storing objects in destFile.
  Example::
    from soma.minf.api import createMinfWriter
    writer = createMinfWriter( '/tmp/test.minf' )
    writer.write( 'A string' )
    writer.write( { 'A dict key': [ 'A list', 'with two elements' ] } )
    writer.close()
  @param format: name of the format to write.
  @type  format: string
  @param reducer: name of the reducer to use (see L{soma.minf.tree} for
    more information about reducers).
  @type  reducer: string
  '''
  return MinfWriter.createWriter( destFile, format, reducer )


#------------------------------------------------------------------------------
def writeMinf( destFile, args, format='XML', reducer=None ):
  '''
  Creates a minf writer with L{createMinfWriter} and write the content
  of C{args} in it.

  @param destFile: see L{createMinfWriter}
  @param args: series of values to write
  @type  args: sequence or iterator
  @param format: see L{createMinfWriter}
  @param reducer: see L{createMinfWriter}
  '''
  it = iter( args )
  try:
    firstItem = it.next()
  except StopIteration:
    firstItem = Undefined
  if reducer is None:
    if firstItem is not Undefined:
      reducer = MinfReducer.defaultReducer( firstItem )
      if reducer is None:
        reducer = 'minf_2.0'
      
  writer = createMinfWriter( destFile, format, reducer )
  if firstItem is not Undefined:
    writer.write( firstItem )
    for item in it:
      writer.write( item )
  writer.close()


#------------------------------------------------------------------------------
# xml_reader and xml_writer are not used directly but importing
# them register the XML minf format
import soma.minf.xml_reader
import soma.minf.xml_writer


#------------------------------------------------------------------------------
from soma.minf.xhtml import XHTML
from soma.uuid import Uuid

minf_2_0_reducer = MinfReducer( 'minf_2.0' )
minf_2_0_reducer.registerAtomType( None.__class__ )
minf_2_0_reducer.registerAtomType( bool )
minf_2_0_reducer.registerAtomType( int )
minf_2_0_reducer.registerAtomType( long )
minf_2_0_reducer.registerAtomType( float )
minf_2_0_reducer.registerAtomType( str )
minf_2_0_reducer.registerAtomType( unicode )
minf_2_0_reducer.registerAtomType( XHTML )
minf_2_0_reducer.registerClass( list, minf_2_0_reducer.sequenceReducer )
minf_2_0_reducer.registerClass( tuple, minf_2_0_reducer.sequenceReducer )
minf_2_0_reducer.registerClass( dict, minf_2_0_reducer.dictReducer )

minf_2_0_expander = MinfExpander( 'minf_2.0' )
minf_2_0_expander.registerStructure( listStructure, minf_2_0_expander.sequenceExpander )
minf_2_0_expander.registerStructure( dictStructure, minf_2_0_expander.dictExpander )

registerClass( 'minf_2.0', Uuid, 'Uuid' )


#------------------------------------------------------------------------------
minf_1_0_reducer = MinfReducer( 'minf_1.0', ( 'minf_2.0', )  )
minf_1_0_expander = MinfExpander( 'minf_1.0', ( 'minf_2.0', )  )



# -*- coding: utf-8 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL license version 2 under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the 
# terms of the CeCILL license version 2 as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license version 2 and that you accept its terms.

'''
QLineEditModificationTimer and TimeredQLineEdit classes associate a
QtCore.QTimer to a QtGui.QLineEdit in order to signal user
modification only after an inactivity period.

author: Yann Cointepas
organization: `NeuroSpin <http://www.neurospin.org>`_ and `IFR 49 http://www.ifr49.org>`_
license: `CeCILL version 2 <http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>`_
'''
__docformat__ = "epytext en"

from soma.qt_gui.qt_backend import QtCore, QtGui

#-------------------------------------------------------------------------------
class QLineEditModificationTimer( QtCore.QObject ):
  '''
  A QLineEditModificationTimer instance is accociated to a
  QtGui.QLineEdit instance, it listens all user modification (Qt
  signal 'textChanged( const QString & )') and emits a
  signal 'userModification()' when timerInterval milliseconds passed
  since the last user modification.
  '''
  # Default timer interval in milliseconds
  defaultTimerInterval = 2000
  userModification = QtCore.Signal()

  def __init__( self, qLineEdit, timerInterval=None ):
    '''
    Parameters
    ----------
    qLineEdit: (QtGui.QLineEdit instance)
        widget associated with this QLineEditModificationTimer.
    timerInterval: (milliseconds)
        minimum inactivity period before emitting
        userModification signal. Default value is
        QLineEditModificationTimer.defaultTimerInterval

    see: TimeredQLineEdit
    '''

    QtCore.QObject.__init__( self )
    # QLineEdit<qt.QLineEdit> instance associated with this
    # QLineEditModificationTimer
    self.qLineEdit = qLineEdit
    if timerInterval is None:
      self.timerInterval = self.defaultTimerInterval
    else:
      # minimum inactivity period before emitting C{userModification} signal.
      self.timerInterval = timerInterval
    self.__timer = QtCore.QTimer( self )
    self.__timer.setSingleShot(True)
    self.__internalModification = False
    self.qLineEdit.textChanged.connect(self._userModification)
    #self.qLineEdit.lostFocus.connect(self._noMoreUserModification)
    self.qLineEdit.editingFinished.connect(self._noMoreUserModification)
    self.__timer.timeout.connect(self.modificationTimeout)

  
  def close( self ):
    self.stop()
    self.qLineEdit.textChanged.disconnect(self._userModification)
    #self.qLineEdit.lostFocus.disconnect(self._noMoreUserModification)
    self.qLineEdit.editingFinished.disconnect(self._noMoreUserModification)
    self.__timer.timeout.disconnect(self.modificationTimeout)
    
    
  def _userModification( self, value ):
    if not self.__internalModification:
      self.__timer.start( self.timerInterval )

  def modificationTimeout(self):
    self.userModification.emit() #self.qLineEdit.text())

  def _noMoreUserModification( self ):
    if self.__timer.isActive():
      self.__timer.stop()
      self.userModification.emit() #self.qLineEdit.text())


  def stopInternalModification( self ):
    '''
    Stop emitting C{userModification} signal when associated
    L{QLineEdit<qt.QLineEdit>} is modified.
    
    @see: L{startInternalModification}
    '''
    self.__internalModification = False


  def startInternalModification( self ):
    '''
    Restart emitting C{userModification} signal when associated
    L{QLineEdit<qt.QLineEdit>} is modified.
    
    @see: L{stopInternalModification}
    '''
    self.__internalModification = True


  def stop( self ):
    '''
    Stop the timer if it is active.
    '''
    self.__timer.stop()

  
  def isActive( self ):
    '''
    Returns True if the timer is active, or False otherwise.
    '''
    return self.__timer.isActive()


#-------------------------------------------------------------------------------
class TimeredQLineEdit( QtGui.QLineEdit ):
  '''
  Create a L{QLineEdit<QtGui.QLineEdit>} instance that has an private attribute
  containing a L{QLineEditModificationTimer} associated to C{self}. Whenever
  the internal L{QLineEditModificationTimer} emits a C{SIGNAL( 
  'userModification' )} signal, this signal is also emited by the 
  L{TimeredQLineEdit} instance.
  '''

  userModification = QtCore.Signal()

  def __init__( self, *args, **kwargs ):
    '''
    All non keyword parameters of the constructor are passed to
    L{QLineEdit<QtGui.QLineEdit>} constructor. An optional C{timerInterval} 
    keyword parameter can be given, it is passed to 
    L{QLineEditModificationTimer} constructor. At the time this class was 
    created, L{QLineEdit<qt.QLineEdit>} constructor did not accept keyword 
    parameters.
    '''
    timerInterval = kwargs.pop( 'timerInterval', None )
    if kwargs:
      QtGui.QLineEdit.__init__( self, *args, **kwargs )
    else:
      QtGui.QLineEdit.__init__( self, *args )
    self.__timer = QLineEditModificationTimer( self,
                                               timerInterval=timerInterval )
    self.__timer.userModification.connect(self.userModification)


  def stopInternalModification( self ):
    '''
    @see: L{QLineEditModificationTimer.stopInternalModification}
    '''
    self.__timer.stopInternalModification()


  def startInternalModification( self ):
    '''
    @see: L{QLineEditModificationTimer.startInternalModification}
    '''
    self.__timer.startInternalModification()

  def close( self ):
    self.__timer.close()
    super( TimeredQLineEdit, self ).close()


# -*- coding: utf-8 -*-
from soma.qt_gui.qt_backend import QtCore, QtGui
from soma.qt_gui.qt_backend.QtCore import QObject, SIGNAL
from soma.qt_gui.qt_backend.QtGui import QWidget, QVBoxLayout

from soma.application import Application
from soma.controller import trait_ids


class WidgetToControllerConnector(object):

    @staticmethod
    def widget_ids(widget):
        return [klass.__name__ for klass in widget.__class__.__mro__
                if issubclass(klass, QWidget)]

    default_factory = {}

    def __init__(self):
        self.factory = {}

    def get_factory(self, trait_id, widget_id):
        key = (trait_id, widget_id)
        factory = self.factory.get(key)
        if factory is None:
            factory = self.default_factory.get(key)
        return factory

    def connect(self, controller, widget):
        # print '!connect!', controller, widget
        for trait_name in controller.trait_names():
            # print '!connect! 2', trait_name
            subwidget = getattr(widget, trait_name, None)
            if isinstance(subwidget, QWidget):
                # print '!connect! 3', subwidget
                for trait_id in trait_ids(controller.trait(trait_name)):
                    for widget_id in self.widget_ids(subwidget):
                        # print '!connect! 4', trait_id, widget_id
                        connector_factory = self.get_factory(
                            trait_id, widget_id)
                        if connector_factory is not None:
                            connector = connector_factory(
                                controller,
                                trait_name,
                                subwidget)
                            # print '!connect! 5', connector
                            break


class TraitWidgetConnector(QObject):

    def __init__(self, controller, attribute_name, widget):
        super(QObject, self).__init__(widget)
        self.controller = controller
        self.attribute_name = attribute_name
        self._ignore_update = False
        self.connect_widget()
        self.controller.on_trait_change(
            self._update_widget,
            self.attribute_name)
        self.update_widget()

    def __del__(self):
        self.disconnect_widget()
        self.controller.on_trait_change(
            self._update_widget,
            self.attribute_name,
            remove=True)

    def _update_widget(self):
        if self._ignore_update == False:
            self._ignore_update = True
            try:
                self.update_widget()
            finally:
                self._ignore_update = False


class Connector_Unicode_QLineEdit(TraitWidgetConnector):
    unicode_to_attribute = lambda x: x
    attribute_to_unicode = unicode

    def connect_widget(self):
        self.parent().textChanged.connect(self.update_controller)

    def disconnect_widget(self):
        try:
            self.parent().textChanged.disconnect(self.update_controller)
        except RuntimeError:
            pass

    def update_widget(self):
        self.parent().setText(self.attribute_to_unicode(
            getattr(self.controller, self.attribute_name)))

    def update_controller(self, value):
        setattr(
            self.controller,
            self.attribute_name,
            self.unicode_to_attribute(self.parent().text()))

WidgetToControllerConnector.default_factory[
    ('Unicode', 'QLineEdit')] = Connector_Unicode_QLineEdit
WidgetToControllerConnector.default_factory[
    ('String', 'QLineEdit')] = Connector_Unicode_QLineEdit
WidgetToControllerConnector.default_factory[
    ('Str', 'QLineEdit')] = Connector_Unicode_QLineEdit


class Connector_Float_QLineEdit(Connector_Unicode_QLineEdit):
    unicode_to_attribute = float
WidgetToControllerConnector.default_factory[
    ('Float', 'QLineEdit')] = Connector_Float_QLineEdit


class Connector_Int_QLineEdit(Connector_Unicode_QLineEdit):
    unicode_to_attribute = int

WidgetToControllerConnector.default_factory[
    ('Int', 'QLineEdit')] = Connector_Int_QLineEdit


class Connector_Bool_QCheckBox(TraitWidgetConnector):

    def connect_widget(self):
        self.parent().stateChanged.connect(self.update_controller)

    def disconnect_widget(self):
        self.parent().stateChanged.disconnect(self.update_controller)

    def update_widget(self):
        self.parent().setChecked(
            getattr(self.controller, self.attribute_name))

    def update_controller(self, value):
        setattr(
            self.controller,
            self.attribute_name,
            self.parent().isChecked())

WidgetToControllerConnector.default_factory[
    ('Bool', 'QCheckBox')] = Connector_Bool_QCheckBox


class Connector_ListFloat_QLineEdit(TraitWidgetConnector):

    def __init__(self, controller, attribute_name, widget):
        super(
            Connector_ListFloat_QLineEdit,
            self).__init__(controller,
                           attribute_name,
                           widget)
        self.controller.on_trait_change(
            self._update_widget,
            attribute_name + '_items')

    def connect_widget(self):
        self.parent().textChanged.connect(self.update_controller)

    def disconnect_widget(self):
        self.parent().textChanged.disconnect(self.update_controller)

    def update_widget(self):
        self.parent().setText(' '.join((str(i)
                                        for i in getattr(self.controller,
                                                         self.attribute_name))))

    def update_controller(self, value):
        setattr(
            self.controller,
            self.attribute_name,
            [float(i) for i in self.parent().text().split()])

WidgetToControllerConnector.default_factory[
    ('List_Float', 'QLineEdit')] = Connector_ListFloat_QLineEdit


class Connector_Float_QSlider(TraitWidgetConnector):

    def connect_widget(self):
        self.parent().valueChanged.connect(self.update_controller)

    def disconnect_widget(self):
        self.parent().valueChanged.disconnect(self.update_controller)

    def update_widget(self):
        self.parent().setValue(getattr(self.controller, self.attribute_name))

    def update_controller(self, value):
        setattr(self.controller, self.attribute_name, self.parent().value())

WidgetToControllerConnector.default_factory[
    ('Float', 'QSlider')] = Connector_Float_QSlider


class Connector_object_QFrame(TraitWidgetConnector):

    def connect_widget(self):
        self.layout = QVBoxLayout(self.parent())
        self.subwidget = Application().gui.create_widget(
            getattr(self.controller, self.attribute_name), parent=self.parent(),
            live=True)
        if self.subwidget is not None:
            self.layout.addWidget(self.subwidget)

    def disconnect_widget(self):
        pass

    def update_widget(self):
        pass

    def update_controller(self):
        pass

WidgetToControllerConnector.default_factory[
    ('Instance_object', 'QFrame')] = Connector_object_QFrame



#! /usr/bin/env python
##########################################################################
# Caspul - Copyright (C) CEA, 2013
# Distributed under the terms of the CeCILL-B license, as published by
# the CEA-CNRS-INRIA. Refer to the LICENSE file or to
# http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
# for details.
##########################################################################

# System import
import logging

# Define the logger
logger = logging.getLogger(__name__)

# Trait import
from traits.api import HasTraits, Event, CTrait

# Soma import
from soma.sorted_dictionary import SortedDictionary


class ControllerMeta(HasTraits.__metaclass__):
    """ This metaclass allows for automatic registration of factories.
    """
    def __new__(mcs, name, bases, dictionary):
        """ Method that can be used to define factories.

        Parameters
        ----------
        mcls: meta class (mandatory)
            a meta class.
        name: str (mandatory)
            the controller class name.
        bases: tuple (mandatory)
            the direct base classes.
        attrs: dict (mandatory)
            a dictionnary with the class attributes.
        """
        return super(ControllerMeta, mcs).__new__(mcs, name, bases, dictionary)


class Controller(HasTraits):
    """ A Controller contains some traits: attributes typing and observer
    (callback) pattern.

    The class provides some methods to add/remove/inspect user defined traits.

    Attributes
    ----------
    `user_traits_changed` : Event
        single event that can be send when several traits changes. This event
        have to be triggered explicitely to take into account changes due to
        call(s) to add_trait or remove_trait.

    Methods
    -------
    user_traits
    is_user_trait
    add_trait
    remove_trait
    _clone_trait
    """
    # Meta class used to defined factories
    __metaclass__ = ControllerMeta

    # This event is necessary because there is no event when a trait is
    # removed with remove_trait and because it is sometimes better to send
    # a single event when several traits changes are done (especially
    # when GUI is updated on real time). This event have to be triggered
    # explicitely to take into account changes due to call(s) to
    # add_trait or remove_trait.
    user_traits_changed = Event

    def __init__(self, *args, **kwargs):
        """ Initilaize the Controller class.

        During the class initialization create a class attribute
        '_user_traits' that contains all the class traits and instance traits
        defined by user (i.e.  the traits that are not automatically
        defined by HasTraits or Controller). We can access this class
        parameter with the 'user_traits' method.

        If user trait parameters are defined directly on derived class, this
        procedure call the 'add_trait' method in order to not share
        user traits between instances.
        """
        # Inheritance
        super(Controller, self).__init__(*args, **kwargs)

        # Create a sorted dictionnary with user parameters
        # The dictionary order correspond to the definition order
        self._user_traits = SortedDictionary()

        # Get all the class traits
        class_traits = self.class_traits()

        # If some traits are defined on the controller, create a list
        # with definition ordered trait name. These names will correspond
        # to user trait sorted dictionary keys
        if class_traits:
            sorted_names = sorted(
                (getattr(trait, "order", ""), name)
                for name, trait in class_traits.iteritems()
                if self.is_user_trait(trait))
            sorted_names = [sorted_name[1] for sorted_name in sorted_names]

            # Go through all trait names that have been ordered
            for name in sorted_names:

                # If the trait is defined on the class, need to clone
                # the class trait and add the cloned trait to the instance.
                # This step avoids us to share trait objects between
                # instances.
                if name in self.__base_traits__:
                    logger.debug("Add class parameter '{0}'.".format(name))
                    trait = class_traits[name]
                    self.add_trait(name, self._clone_trait(trait))

                # If the trait is defined on the instance, just
                # add the user parameter to the '_user_traits' instance
                # parameter
                else:
                    logger.debug("Add instance parameter '{0}'.".format(name))
                    self._user_traits[name] = class_traits[name]

    ####################################################################
    # Private methods
    ####################################################################

    def _clone_trait(self, clone, metadata=None):
        """ Creates a clone of a specific trait (ie. the same trait
        type but different ids).

        Parameters
        ----------
        clone: CTrait (mandatory)
            the input trait to clone.
        metadata: dict (opional, default None)
            some metadata than can be added to the trait __dict__.

        Returns
        -------
        trait: CTrait
            the cloned input trait.
        """
        # Create an empty trait
        trait = CTrait(0)

        # Clone the input trait in the empty trait structure
        trait.clone(clone)

        # Set the input trait __dict__ elements to the cloned trait
        # __dict__
        if clone.__dict__ is not None:
            trait.__dict__ = clone.__dict__.copy()

        # Update the cloned trait __dict__ if necessary
        if metadata is not None:
            trait.__dict__.update(metadata)

        return trait

    ####################################################################
    # Public methods
    ####################################################################

    def user_traits(self):
        """ Methood to access the user parameters.

        Returns
        -------
        out: dict
            a dictionnary containing class traits and instance traits
            defined by user (i.e.  the traits that are not automatically
            defined by HasTraits or Controller). Returned values are
            sorted according to the 'order' trait meta-attribute.
        """
        return self._user_traits

    def is_user_trait(self, trait):
        """ Method that evaluate if a trait is a user parameter
        (i.e. not an Event).

        Returns
        -------
        out: bool
            True if the trait is a user trait,
            False otherwise.
        """
        return not isinstance(trait.handler, Event)

    def add_trait(self, name, *trait):
        """ Add a new trait.

        Parameters
        ----------
        name: str (mandatory)
            the trait name.
        trait: traits.api (mandatory)
            a valid trait.
        """
        # Debug message
        logger.debug("Adding trait '{0}'...".format(name))

        # Inheritance: create the instance trait attribute
        super(Controller, self).add_trait(name, *trait)

        # Get the trait instance and if it is a user trait load the traits
        # to get it direcly from the instance (as a property) and add it
        # to the class '_user_traits' attributes
        trait_instance = self.trait(name)
        if self.is_user_trait(trait_instance):
            trait_instance.defaultvalue = trait_instance.default
            self.get(name)
            self._user_traits[name] = trait_instance

    def remove_trait(self, name):
        """ Remove a trait from its name.

        Parameters
        ----------
        name: str (mandatory)
            the trait name to remove.
        """
        # Debug message
        logger.debug("Removing trait '{0}'...".format(name))

        # Call the Traits remove_trait method
        super(Controller, self).remove_trait(name)

        # Remove name from the '_user_traits' without error if it
        # is not present
        self._user_traits.pop(name, None)



#! /usr/bin/env python
##########################################################################
# NSAp - Copyright (C) CEA, 2013
# Distributed under the terms of the CeCILL-B license, as published by
# the CEA-CNRS-INRIA. Refer to the LICENSE file or to
# http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
# for details.
##########################################################################

from ez_setup import use_setuptools
use_setuptools()

import os
from setuptools import setup, find_packages
from datetime import datetime

release_info = {}
execfile(os.path.join("soma", "info.py"), release_info)

setup(
    name=release_info["NAME"],
    description=release_info["DESCRIPTION"],
    long_description=release_info["LONG_DESCRIPTION"],
    license=release_info["LICENSE"],
    classifiers=release_info["CLASSIFIERS"],
    author=release_info["AUTHOR"],
    author_email=release_info["AUTHOR_EMAIL"],
    version=release_info["VERSION"],
    packages=find_packages(),
    platforms=release_info["PLATFORMS"],
    #scripts=["../../bin/bv_minf_2_XML", ]
)



# -*- coding: utf-8 -*-
import sipconfig
from soma.qt_gui.qt_backend import QtGui, QtCore
from soma.controller import trait_ids, Controller
from soma.functiontools import partial, SomaPartial
from trait_to_widget import StrCreateWidget, FloatCreateWidget, \
    IntCreateWidget, LongCreateWidget, BoolCreateWidget, EnumCreateWidget, \
    StrEnumCreateWidget, FileCreateWidget, DirectoryCreateWidget
from soma.gui.icon_factory import IconFactory
import weakref


class ControllerWidget(QtGui.QWidget):
    # QPixmap instance cannot be build before a QApplication is being created.
    # (it simply exits the program). But for documentation with epydoc, all
    # modules are loaded without C{QApplication}. Therefore, the creation of
    # the following two static QPixmap instances has been put in __init__.

    _create_widget = {}

    def __init__(self, controller,
                 parent=None, name=None, live=False, hide_labels=False):

        super(ControllerWidget, self).__init__(parent)
        if name:
            self.setObjectName(name)
        self.controller = controller
        self.live = live
        self.connected = False
        self.hide_labels = hide_labels
        self._grid_layout = QtGui.QGridLayout()
        self._grid_layout.setAlignment(QtCore.Qt.AlignTop)
        self._grid_layout.setSpacing(3)
        self._grid_layout.setContentsMargins(5, 5, 5, 5)
        self.setLayout(self._grid_layout)

        self.btn_expand = None
        self._widgets = {}
        self._create_widgets()
        if self.live:
            self.connect_controller()
        else:
            self.update_controller_widget()

    def __del__(self):
        self.disconnect_controller()

    def update_controller(self):
        """Update traits when GUI change"""
        for name, tuple in self._widgets.iteritems():
            trait, create_widget, attribute_widget, label_widget = tuple
            create_widget.update_controller(self, name, attribute_widget)

    def update_controller_widget(self):
        """Update GUI when traits change"""
        for name, tuple in self._widgets.iteritems():
            trait, create_widget, attribute_widget, label_widget = tuple
            create_widget.update_controller_widget(
                self,
                name,
                attribute_widget,
                label_widget)

    # def back_default_value(self,attribute_widget,name):
            # print 'here back default value'
            # print self.controller.trait(name)
            # print self.controller.trait(name).defaultvalue
            # setattr( self.controller, name,
                # self.controller.trait(name).defaultvalue )
            # attribute_widget.btn_default_value.setVisible(False)
    def connect_controller(self):
        """
        To connect traits and GUI, use update_controller and
        update controller_widget
        """
        if not self.connected:
            for name, tupl in self._widgets.iteritems():
                trait, create_widget, attribute_widget, label_widget = tupl
                create_widget.connect_controller(
                    self,
                    name,
                    attribute_widget,
                    label_widget)
            self.controller.on_trait_change(
                self.update_widgets,
                'user_traits_changed')
            self.update_controller_widget()
            self.connected = True

    def disconnect_controller(self):
        """To disconnect traits and GUI"""
        if self.connected:
            self.controller.on_trait_change(
                self.update_widgets,
                'user_traits_changed',
                remove=True)
            for name, tupl in self._widgets.iteritems():
                trait, create_widget, attribute_widget, label_widget = tupl
                create_widget.disconnect_controller(
                    self,
                    name,
                    attribute_widget,
                    label_widget)
            self.connected = False

    @classmethod
    def find_create_widget_from_trait(cls, trait):
        """
        Find widget corresponding to the trait, call the corresponding
        create_create widget class
        """
        create_widget = None
        for trait_id in trait_ids(trait):
            for create_widget in cls._create_widget.get(trait_id, []):
                if create_widget.is_valid_trait(trait):
                    break
            else:
                create_widget = None
            if create_widget is not None:
                break
        return create_widget

    def _create_widgets(self):
        # print '!_create_widgets!', self.controller.user_traits().keys()
        for name, trait in self.controller.user_traits().iteritems():
            create_widget = self._widgets.get(name)
            if create_widget is None:
                create_widget = self.find_create_widget_from_trait(trait)
                if not create_widget:
                    continue
                attribute_widget, label_widget = create_widget.create_widget(
                    self, name, trait, getattr(self.controller, name))
                if trait.desc:
                    tooltip = '<b>' + name + ':</b> ' + trait.desc
                else:
                    tooltip = None
                if label_widget is None:
                    self._grid_layout.addWidget(
                        attribute_widget,
                        self._grid_layout.rowCount(),
                        0,
                        1,
                        2)
                    if tooltip:
                        attribute_widget.setToolTip(tooltip)
                elif isinstance(label_widget, tuple):
                    row = self._grid_layout.rowCount()
                    if label_widget[0] and not self.hide_labels:
                        self._grid_layout.addWidget(label_widget[0], row, 0)
                        if tooltip:
                            label_widget[0].setToolTip(tooltip)
                    self._grid_layout.addWidget(label_widget[1], row, 1)
                    if tooltip:
                        label_widget[1].setToolTip(tooltip)
                    self._grid_layout.addWidget(
                        attribute_widget,
                        self._grid_layout.rowCount(),
                        0,
                        1,
                        2)
                else:
                    row = self._grid_layout.rowCount()
                    if self.hide_labels:
                        self._grid_layout.addWidget(
                            attribute_widget,
                            row,
                            0,
                            1,
                            2)
                        if tooltip:
                            attribute_widget.setToolTip(tooltip)
                    else:
                        self._grid_layout.addWidget(label_widget, row, 0)
                        self._grid_layout.addWidget(attribute_widget, row, 1)
                        # btn_default_value = QtGui.QPushButton()
                        # btn_default_value.setIcon( QtGui.QIcon(
                            # IconFactory()._valueModified) )
                        # btn_default_value.setVisible(False)
                        # attribute_widget.btn_default_value = btn_default_value
                        # self._grid_layout.addWidget( btn_default_value,row,2
                        # )
                        if tooltip:
                            label_widget.setToolTip(tooltip)
                            attribute_widget.setToolTip(tooltip)
                self._widgets[name] = (
                    trait,
                    create_widget,
                    attribute_widget,
                    label_widget)
            else:
                trait, create_widget, attribute_widget, label_widget \
                    = create_widget
            if getattr(trait, 'hidden', False):
                attribute_widget.hide()
                if label_widget:
                    if isinstance(label_widget, tuple):
                        l = label_widget
                    else:
                        l = [label_widget]
                    for i in l:
                        i.hide()
            else:
                attribute_widget.show()
                if label_widget:
                    if isinstance(label_widget, tuple):
                        l = label_widget
                    else:
                        l = [label_widget]
                    for i in l:
                        i.show()

    def update_widgets(self):
        user_traits = self.controller.user_traits()
        was_connected = self.connected
        if was_connected:
            self.disconnect_controller()
        for name in self._widgets.keys():
            trait, create_widget, attribute_widget, label_widget \
                = self._widgets[name]
            if user_traits.get(name) != trait:
                attribute_widget.close()
                attribute_widget.deleteLater()
                if isinstance(label_widget, tuple):
                    if label_widget[0]:
                        label_widget[0].close()
                        label_widget[0].deleteLater()
                    if label_widget[1]:
                        label_widget[1].close()
                        label_widget[1].deleteLater()
                elif label_widget:
                    label_widget.close()
                    label_widget.deleteLater()
                del self._widgets[name]
        self._create_widgets()
        if was_connected:
            self.connect_controller()
        self.updateGeometry()


#-------------------------------------------------------------------------
class ListCreateWidget(object):

    class ListController(Controller):
        pass

    @staticmethod
    def is_valid_trait(trait):
        item_trait = trait.inner_traits[0]
        create_widget = ControllerWidget.find_create_widget_from_trait(
            item_trait)
        return create_widget is not None

    @classmethod
    def create_widget(cls, parent, name, trait, value):
        item_trait = trait.inner_traits[0]
        # print '!ListCreateWidget!', name, 'List( %s )' % trait_ids(
        # item_trait )
        list_controller = cls.ListController()
        for i in xrange(len(value)):
            list_controller.add_trait(str(i), item_trait)
            trait = list_controller.trait(str(i))
            trait.order = i
            setattr(list_controller, str(i), value[i])
        result = ControllerCreateWidget.create_widget(
            parent, name, None, list_controller)
        attribute_widget = result[0]
        attribute_widget.controller_widget.connect_controller()
        attribute_widget.item_trait = item_trait
        attribute_widget.list_controller = list_controller
        attribute_widget.connected = False
        return result

    @staticmethod
    def update_controller(controller_widget, name, attribute_widget):
        items = [getattr(attribute_widget.list_controller, str(i))
                 for i in
                 xrange(len(attribute_widget.list_controller.user_traits()))]
        # print '!update_controller!', name, len( items ), items
        setattr(controller_widget.controller, name, items)

    @classmethod
    def update_controller_widget(
            cls, controller_widget, name, attribute_widget, label_widget):
        was_connected = attribute_widget.connected
        cls.disconnect_controller(
            controller_widget,
            name,
            attribute_widget,
            label_widget)
        attribute_widget.controller_widget.disconnect_controller()
        items = getattr(controller_widget.controller, name)
        len_widget = len(attribute_widget.list_controller.user_traits())
        # print '!update_controller_widget!', name, len_widget, items
        user_traits_changed = False
        if len(items) < len_widget:
            for i in xrange(len(items), len_widget):
                attribute_widget.list_controller.remove_trait(str(i))
            user_traits_changed = True
        elif len(items) > len_widget:
            for i in xrange(len_widget, len(items)):
                attribute_widget.list_controller.add_trait(
                    str(i), attribute_widget.item_trait)
                trait = attribute_widget.list_controller.trait(str(i))
                trait.order = i
            user_traits_changed = True
        for i in xrange(len(items)):
            setattr(attribute_widget.list_controller, str(i), items[i])
        # print '!update_controller_widget! done', name
        attribute_widget.controller_widget.connect_controller()
        if user_traits_changed:
            attribute_widget.list_controller.user_traits_changed = True
        if was_connected:
            cls.connect_controller(
                controller_widget,
                name,
                attribute_widget,
                label_widget)

    @classmethod
    def connect_controller(
            cls, controller_widget, name, attribute_widget, label_widget):
        if not attribute_widget.connected:
            def list_controller_hook(obj, key, old, new):
                # print '!list_controller_hook!', ( obj, key, old, new )
                items = getattr(controller_widget.controller, name)
                items[int(key)] = new
            for n in attribute_widget.list_controller.user_traits():
                attribute_widget.list_controller.on_trait_change(
                    list_controller_hook, n)
            controller_hook = SomaPartial(
                cls.update_controller_widget,
                weakref.proxy(controller_widget), name, attribute_widget,
                label_widget)
            controller_widget.controller.on_trait_change(
                controller_hook, name + '[]')
            attribute_widget._controller_connections = (
                list_controller_hook, controller_hook)
            attribute_widget.connected = True

    @staticmethod
    def disconnect_controller(
            controller_widget, name, attribute_widget, label_widget):
        if attribute_widget.connected:
            list_controller_hook, controller_hook \
                = attribute_widget._controller_connections
            controller_widget.controller.on_trait_change(
                controller_hook, name + '[]', remove=True)
            for n in attribute_widget.list_controller.user_traits():
                attribute_widget.list_controller.on_trait_change(
                    list_controller_hook, n, remove=True)
            del attribute_widget._controller_connections
            attribute_widget.connected = False


#-------------------------------------------------------------------------
class ControllerCreateWidget(object):

    """Class for instance trait"""
    @staticmethod
    def is_valid_trait(trait):
        return True

    @classmethod
    def create_widget(cls, parent, name, trait, sub_controller):
        label = getattr(trait, 'label', None)
        if not label:
            label = name
        if label is not None:
            label_widget = QtGui.QLabel(label, parent)
        else:
            label_widget = None

        btn_expand = QtGui.QPushButton(parent)
        btn_expand.setIcon(QtGui.QIcon(IconFactory()._imageExpand))

        scroll_area = QtGui.QScrollArea(parent=parent)
        scroll_area.setWidgetResizable(True)
        # scroll_area.setSizePolicy( QtGui.QSizePolicy(
            # QtGui.QSizePolicy.Expanding,
            # QtGui.QSizePolicy.Expanding
            # ) )
        sub_controller_widget = ControllerWidget(sub_controller,
                                                 parent=scroll_area)
        scroll_area.setWidget(sub_controller_widget)
        # sub_controller_widget.setSizePolicy( QtGui.QSizePolicy(
            # QtGui.QSizePolicy.Expanding,
            # QtGui.QSizePolicy.MinimumExpanding
            # ) )
        scroll_area.setFrameShape(QtGui.QFrame.StyledPanel)
        expand_hook = partial(cls.expand_collapse, scroll_area)
        btn_expand.connect(btn_expand, QtCore.SIGNAL('clicked()'),
                           expand_hook)
        attribute_widget = scroll_area
        attribute_widget.btn_expand = btn_expand
        attribute_widget.label_widget = label_widget
        attribute_widget.expand_hook = expand_hook
        scroll_area.hide()
        attribute_widget.controller_widget = sub_controller_widget
        return (attribute_widget, (label_widget, btn_expand))

    @staticmethod
    def update_controller(controller_widget, name, attribute_widget):
        attribute_widget.controller_widget.update_controller()

    @staticmethod
    def update_controller_widget(
            controller_widget, name, attribute_widget, label_widget):
        attribute_widget.controller_widget.update_controller_widget()

    @staticmethod
    def connect_controller(
            controller_widget, name, attribute_widget, label_widget):
        attribute_widget.controller_widget.connect_controller()

    @staticmethod
    def disconnect_controller(
            controller_widget, name, attribute_widget, label_widget):
        attribute_widget.controller_widget.disconnect_controller()

    @staticmethod
    def expand_collapse(attribute_widget):
        if attribute_widget.isVisible():
            attribute_widget.hide()
            attribute_widget.btn_expand.setIcon(
                QtGui.QIcon(IconFactory()._imageExpand))
        else:
            attribute_widget.show()
            attribute_widget.btn_expand.setIcon(
                QtGui.QIcon(IconFactory()._imageCollapse))


ControllerWidget._create_widget['Str'] = [StrCreateWidget]
ControllerWidget._create_widget['Unicode'] = [StrCreateWidget]
ControllerWidget._create_widget['Float'] = [FloatCreateWidget]
ControllerWidget._create_widget['Int'] = [IntCreateWidget]

ControllerWidget._create_widget['Long'] = [LongCreateWidget]
ControllerWidget._create_widget['Bool'] = [BoolCreateWidget]

ControllerWidget._create_widget['Enum'] = [EnumCreateWidget]
ControllerWidget._create_widget['Str'].insert(0, StrEnumCreateWidget)
ControllerWidget._create_widget['Unicode'].insert(0, StrEnumCreateWidget)
ControllerWidget._create_widget['File'] = [FileCreateWidget]
ControllerWidget._create_widget['Directory'] = [DirectoryCreateWidget]
ControllerWidget._create_widget[
    'Instance_Controller'] = [
    ControllerCreateWidget]
ControllerWidget._create_widget['Any'] = [StrCreateWidget]

ControllerWidget._create_widget['List_Str'] = [ListCreateWidget]
ControllerWidget._create_widget['List_Float'] = [ListCreateWidget]
ControllerWidget._create_widget['List_Int'] = [ListCreateWidget]
ControllerWidget._create_widget['List_Long'] = [ListCreateWidget]
ControllerWidget._create_widget['List_Bool'] = [ListCreateWidget]
ControllerWidget._create_widget['List_Enum'] = [ListCreateWidget]
ControllerWidget._create_widget['List_Directory'] = [ListCreateWidget]
ControllerWidget._create_widget['List_File'] = [ListCreateWidget]
ControllerWidget._create_widget['List_Any'] = [ListCreateWidget]


#-------------------------------------------------------------------------------
# if __name__ == '__main__':
    # import sys
    # try:
        # from traits.api import Str, Unicode, Float, Int, Long, Bool, Enum, Directory, File, Instance, List
    # except:
        # from enthought.traits.api import Str, Unicode, Float, Int, Long, Bool, Enum, Directory, File, Instance, List
    # from soma.controller import Controller

    # app = QtGui.QApplication( sys.argv )

    # class InstanceTraits( Controller ):
        # def __init__( self ):
            # self.add_trait( 'str', Str( desc='This is a string', order=1, viewer='viewers.image.ImageViewer' ) )
            # self.add_trait( 'str2', Str( desc='This is a String', order=2 ) )
            # self.add_trait( 'unicode', Str( desc='This is a unicode', order=2 ) )
            # self.add_trait( 'float', Float( desc='This is a float', order=3 ) )
            # self.add_trait( 'int', Int( desc='This is an int', order=4 ) )
            # self.add_trait( 'long', Long( desc='This is a long', order=5 ) )
            # self.add_trait( 'bool', Bool( desc='This is a boolean.', order=6 ) )
            # self.add_trait( 'enum', Enum( 'first value', 'second value', 'third value', desc='This is an enum.', order=7 ) )
            # self.add_trait( 'str_enum', Str( desc='This is a StrEnum', values=( 'first value', 'second value', 'third value' ), order=8 ) )
            # self.add_trait( 'directory', Directory( desc='This is a directory.', order=9 ) )
            # self.add_trait( 'file', File( desc='This is a file.', order=10 )
            # )

            # self.add_trait( 'list_str', List( Str, desc='This is a string list', order=11 ) )
            # self.add_trait( 'list_unicode', List( Unicode, desc='This is a unicode list', order=12 ) )
            # self.add_trait( 'list_float', List( Float, desc='This is a float list', order=13 ) )
            # self.add_trait( 'list_int', List( Int, desc='This is an int list', order=14 ) )
            # self.add_trait( 'list_long', List( Long, desc='This is a long list', order=15 ) )
            # self.add_trait( 'list_bool', List( Bool, desc='This is a boolean list', order=16 ) )
            # self.add_trait( 'list_enum', List( Enum( 'first value', 'second value', 'third value' ), desc='This is an enum list', order=17 ) )
            # self.add_trait( 'list_str_enum', List( Str( values=( 'first value', 'second value', 'third value' ) ), desc='This is a StrEnum list', order=18 ) )
            # self.add_trait( 'list_directory', List( Directory, desc='This is a directory list', order=19 ) )
            # self.add_trait( 'list_file', List( File, desc='This is a file
            # list', order=20 ) )



    # class WidgetOnGui( Controller ):
        # lists_size = Int( 0, order=0, desc='Resize all the lists' )
        # connected = Bool( default_value=True, order=1, desc='Disconnect one of the two widgets' )
        # traits_on_class = Instance( ClassTraits, desc='All traits on this attribute are defined on class', order=2 )
        # traits_on_instance0 = Instance( InstanceTraits, desc='All traits on this attribute are defined on class', order=2 )
        # traits_on_instance = Instance( SimpMorpho, desc='All traits on this
        # attribute are defined on instance', order=3 )


        # def __init__( self ):
            # super( WidgetOnGui, self ).__init__()
            # self.traits_on_class = ClassTraits()
            # self.traits_on_instance0 = InstanceTraits()
            # self.traits_on_instance = SimpMorpho()
            # self.dummy = 'I am hidden'


        # def on_run(self):
            # print 'IN THE FUNCTION RUN'
            # self.traits_on_instance()






    # def resize( main, old, new ):
        # for o in ( main.traits_on_class, main.traits_on_instance ):
        # for o in main.traits_on_instance :
            # for a in ( 'list_str', 'list_unicode', 'list_float', 'list_int', 'list_long', 'list_bool', 'list_enum', 'list_str_enum', 'list_directory', 'list_file' ):
                        # list = getattr( o, a )
                # len_list = len( list )
                # if main.lists_size > len_list:
                    # print '!resize! + ', a, len_list, main.lists_size, list + [ o.trait( a ).inner_traits[0].default ] * ( main.lists_size - len_list )
                    # setattr( o, a, list + [ o.trait( a ).inner_traits[0].default ] * ( main.lists_size - len_list ) )
                    # print '!resize! done'
                # elif main.lists_size < len_list:
                    # print '!resize! - ', a, len_list, main.lists_size
                    # setattr( o, a, list[ : main.lists_size ] )
                    # print '!resize! done'

    # def f( connected ):
        # print '!f!', connected
        # if connected:
            # w1.connect_controller()
        # else:
            # w1.disconnect_controller()
    # o.on_trait_change( f, 'connected' )


    # w2 = ControllerWidget( o, live=True )
    # w2.show()

    # def f(object,name,old,new ):
        # print 'in f'
        # print ( object,name,old,new )

    # o.traits_on_instance.on_trait_change( f )
    # o.on_trait_change( resize, 'lists_size' )
    # app.exec_()


    # w1.on_trait_change(update,'t1mri')

    # def update(object,name,old,new):
            # o.traits_on_instance._t1mri_changed()



# -*- coding: iso-8859-1 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the 
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
This module provides the class L{ThreadSafeProxy}.

@author: Yann Cointepas
@organization: U{NeuroSpin<http://www.neurospin.org>} and U{IFR 49<http://www.ifr49.org>}
@license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''
__docformat__ = "epytext en"

import inspect, threading
import Pyro.core
from soma.thread_calls import SingleThreadCalls
from soma.functiontools import partial

class ThreadSafeProxy( object ):
  _thread = None
  
  def __init__( self, proxy ):
    '''
    This class makes it possible to share Pyro proxy across multiple threads.
    A single Pyro thread is created. This thread will be the only one to use Pyro
    proxy objects. A L{ThreadSafeProxy} is a proxy on a Pyro proxy that will send
    all activity (methods calls and attributes getting and setting) to the
    Pyro thread.
    
    Example
    -------
      import Pyro.core
      from soma.pyro import ThreadSafeProxy
      
      threadSafeProxy = ThreadSafeProxy( Pyro.core.getProxyFromURI( 'PYRO://127.0.0.1:7766/84a68da2260374b1b334d9591c4fd84f' ) )
    '''
    newProxy = self.pyroThreadCall( proxy.__class__, proxy.URI )
    object.__setattr__( self, '_proxy', newProxy )
  
  
  @staticmethod
  def _createThread():
    '''
    Internal method to create the Pyro thread if it does not exists
    '''
    if ThreadSafeProxy._thread is None:
      ThreadSafeProxy._threadCall = SingleThreadCalls()
      ThreadSafeProxy._thread = threading.Thread( target=ThreadSafeProxy._threadCall.processingLoop )
      ThreadSafeProxy._thread.setDaemon( True )
      ThreadSafeProxy._threadCall.setProcessingThread( ThreadSafeProxy._thread )
      ThreadSafeProxy._thread.start()
  
  #_dbg = 0
  @staticmethod
  def pyroThreadCall( *args, **kwargs ):
    '''
    Calls a function from the Pyro thread.
    '''
    ThreadSafeProxy._createThread()
    #n = ThreadSafeProxy._dbg
    #ThreadSafeProxy._dbg += 1
    #if isinstance( f, Pyro.core._RemoteMethod ):
      #sf = '<' + f._RemoteMethod__name + '>'
    #else:
      #sf = repr( f )
    #print '!pyroThreadCall! -->', n, sf, args, kwargs
    #result = ThreadSafeProxy._threadCall.call( f, *args, **kwargs )
    #print '!pyroThreadCall! <--', n, result
    #return result
    return ThreadSafeProxy._threadCall.call( *args, **kwargs )
  
  
  def __getattr__( self, name ):
    value = ThreadSafeProxy.pyroThreadCall( getattr, self._proxy, name )
    if isinstance( value, Pyro.core._RemoteMethod ) or inspect.ismethod( value ):
      return partial( ThreadSafeProxy.pyroThreadCall, value )
    return value


  def __setattr__( self, name, value ):
    return ThreadSafeProxy.pyroThreadCall( setattr, self._proxy, name, value )
